"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.invariant = exports.addErrorToEachTestUnderDescribe = exports.getTestID = exports.makeRunResult = exports.getTestDuration = exports.callAsyncCircusFn = exports.describeBlockHasTests = exports.getEachHooksForTest = exports.getAllHooksForDescribe = exports.makeTest = exports.makeDescribe = void 0;

var _jestUtil = require("jest-util");

var _isGeneratorFn = _interopRequireDefault(require("is-generator-fn"));

var _co = _interopRequireDefault(require("co"));

var _stackUtils = _interopRequireDefault(require("stack-utils"));

var _prettyFormat = _interopRequireDefault(require("pretty-format"));

var _state = require("./state");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Symbol = global['jest-symbol-do-not-touch'] || global.Symbol;
var Symbol = global['jest-symbol-do-not-touch'] || global.Symbol;
var jestNow = global[Symbol.for('jest-native-now')] || global.Date.now;
var Symbol = global['jest-symbol-do-not-touch'] || global.Symbol;
var Promise = global[Symbol.for('jest-native-promise')] || global.Promise;

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }

const stackUtils = new _stackUtils.default({
  cwd: 'A path that does not exist'
});

const makeDescribe = (name, parent, mode) => {
  let _mode = mode;

  if (parent && !mode) {
    // If not set explicitly, inherit from the parent describe.
    _mode = parent.mode;
  }

  return {
    children: [],
    hooks: [],
    mode: _mode,
    name: (0, _jestUtil.convertDescriptorToString)(name),
    parent,
    tests: []
  };
};

exports.makeDescribe = makeDescribe;

const makeTest = (fn, mode, name, parent, timeout, asyncError) => ({
  asyncError,
  duration: null,
  errors: [],
  fn,
  invocations: 0,
  mode,
  name: (0, _jestUtil.convertDescriptorToString)(name),
  parent,
  startedAt: null,
  status: null,
  timeout
}); // Traverse the tree of describe blocks and return true if at least one describe
// block has an enabled test.


exports.makeTest = makeTest;

const hasEnabledTest = describeBlock => {
  const _getState = (0, _state.getState)(),
        hasFocusedTests = _getState.hasFocusedTests,
        testNamePattern = _getState.testNamePattern;

  const hasOwnEnabledTests = describeBlock.tests.some(test => !(test.mode === 'skip' || hasFocusedTests && test.mode !== 'only' || testNamePattern && !testNamePattern.test(getTestID(test))));
  return hasOwnEnabledTests || describeBlock.children.some(hasEnabledTest);
};

const getAllHooksForDescribe = describe => {
  const result = {
    afterAll: [],
    beforeAll: []
  };

  if (hasEnabledTest(describe)) {
    var _iterator = _createForOfIteratorHelper(describe.hooks),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        const hook = _step.value;

        switch (hook.type) {
          case 'beforeAll':
            result.beforeAll.push(hook);
            break;

          case 'afterAll':
            result.afterAll.push(hook);
            break;
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  }

  return result;
};

exports.getAllHooksForDescribe = getAllHooksForDescribe;

const getEachHooksForTest = test => {
  const result = {
    afterEach: [],
    beforeEach: []
  };
  let block = test.parent;

  do {
    const beforeEachForCurrentBlock = [];

    var _iterator2 = _createForOfIteratorHelper(block.hooks),
        _step2;

    try {
      for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
        const hook = _step2.value;

        switch (hook.type) {
          case 'beforeEach':
            beforeEachForCurrentBlock.push(hook);
            break;

          case 'afterEach':
            result.afterEach.push(hook);
            break;
        }
      } // 'beforeEach' hooks are executed from top to bottom, the opposite of the
      // way we traversed it.

    } catch (err) {
      _iterator2.e(err);
    } finally {
      _iterator2.f();
    }

    result.beforeEach = [...beforeEachForCurrentBlock, ...result.beforeEach];
  } while (block = block.parent);

  return result;
};

exports.getEachHooksForTest = getEachHooksForTest;

const describeBlockHasTests = describe => describe.tests.length > 0 || describe.children.some(describeBlockHasTests);

exports.describeBlockHasTests = describeBlockHasTests;

const _makeTimeoutMessage = (timeout, isHook) => `Exceeded timeout of ${timeout}ms for a ${isHook ? 'hook' : 'test'}.\nUse jest.setTimeout(newTimeout) to increase the timeout value, if this is a long-running test.`; // Global values can be overwritten by mocks or tests. We'll capture
// the original values in the variables before we require any files.


const _global = global,
      setTimeout = _global.setTimeout,
      clearTimeout = _global.clearTimeout;

function checkIsError(error) {
  return !!(error && error.message && error.stack);
}

const callAsyncCircusFn = (fn, testContext, {
  isHook,
  timeout
}) => {
  let timeoutID;
  let completed = false;
  return new Promise((resolve, reject) => {
    timeoutID = setTimeout(() => reject(_makeTimeoutMessage(timeout, !!isHook)), timeout); // If this fn accepts `done` callback we return a promise that fulfills as
    // soon as `done` called.

    if (fn.length) {
      const done = reason => {
        const errorAsErrorObject = checkIsError(reason) ? reason : new Error(`Failed: ${(0, _prettyFormat.default)(reason, {
          maxDepth: 3
        })}`); // Consider always throwing, regardless if `reason` is set or not

        if (completed && reason) {
          errorAsErrorObject.message = 'Caught error after test environment was torn down\n\n' + errorAsErrorObject.message;
          throw errorAsErrorObject;
        }

        return reason ? reject(errorAsErrorObject) : resolve();
      };

      return fn.call(testContext, done);
    }

    let returnedValue;

    if ((0, _isGeneratorFn.default)(fn)) {
      returnedValue = _co.default.wrap(fn).call({});
    } else {
      try {
        returnedValue = fn.call(testContext);
      } catch (error) {
        return reject(error);
      }
    } // If it's a Promise, return it. Test for an object with a `then` function
    // to support custom Promise implementations.


    if (typeof returnedValue === 'object' && returnedValue !== null && typeof returnedValue.then === 'function') {
      return returnedValue.then(resolve, reject);
    }

    if (!isHook && returnedValue !== void 0) {
      return reject(new Error(`
      test functions can only return Promise or undefined.
      Returned value: ${String(returnedValue)}
      `));
    } // Otherwise this test is synchronous, and if it didn't throw it means
    // it passed.


    return resolve();
  }).then(() => {
    completed = true; // If timeout is not cleared/unrefed the node process won't exit until
    // it's resolved.

    timeoutID.unref && timeoutID.unref();
    clearTimeout(timeoutID);
  }).catch(error => {
    completed = true;
    timeoutID.unref && timeoutID.unref();
    clearTimeout(timeoutID);
    throw error;
  });
};

exports.callAsyncCircusFn = callAsyncCircusFn;

const getTestDuration = test => {
  const startedAt = test.startedAt;
  return typeof startedAt === 'number' ? jestNow() - startedAt : null;
};

exports.getTestDuration = getTestDuration;

const makeRunResult = (describeBlock, unhandledErrors) => ({
  testResults: makeTestResults(describeBlock),
  unhandledErrors: unhandledErrors.map(_formatError)
});

exports.makeRunResult = makeRunResult;

const makeTestResults = describeBlock => {
  const _getState2 = (0, _state.getState)(),
        includeTestLocationInResult = _getState2.includeTestLocationInResult;

  let testResults = [];

  var _iterator3 = _createForOfIteratorHelper(describeBlock.tests),
      _step3;

  try {
    for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
      const test = _step3.value;
      const testPath = [];
      let parent = test;

      do {
        testPath.unshift(parent.name);
      } while (parent = parent.parent);

      const status = test.status;

      if (!status) {
        throw new Error('Status should be present after tests are run.');
      }

      let location = null;

      if (includeTestLocationInResult) {
        const stackLine = test.asyncError.stack.split('\n')[1];
        const parsedLine = stackUtils.parseLine(stackLine);

        if (parsedLine && typeof parsedLine.column === 'number' && typeof parsedLine.line === 'number') {
          location = {
            column: parsedLine.column,
            line: parsedLine.line
          };
        }
      }

      testResults.push({
        duration: test.duration,
        errors: test.errors.map(_formatError),
        invocations: test.invocations,
        location,
        status,
        testPath
      });
    }
  } catch (err) {
    _iterator3.e(err);
  } finally {
    _iterator3.f();
  }

  var _iterator4 = _createForOfIteratorHelper(describeBlock.children),
      _step4;

  try {
    for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
      const child = _step4.value;
      testResults = testResults.concat(makeTestResults(child));
    }
  } catch (err) {
    _iterator4.e(err);
  } finally {
    _iterator4.f();
  }

  return testResults;
}; // Return a string that identifies the test (concat of parent describe block
// names + test title)


const getTestID = test => {
  const titles = [];
  let parent = test;

  do {
    titles.unshift(parent.name);
  } while (parent = parent.parent);

  titles.shift(); // remove TOP_DESCRIBE_BLOCK_NAME

  return titles.join(' ');
};

exports.getTestID = getTestID;

const _formatError = errors => {
  let error;
  let asyncError;

  if (Array.isArray(errors)) {
    error = errors[0];
    asyncError = errors[1];
  } else {
    error = errors;
    asyncError = new Error();
  }

  if (error) {
    if (error.stack) {
      return error.stack;
    }

    if (error.message) {
      return error.message;
    }
  }

  asyncError.message = `thrown: ${(0, _prettyFormat.default)(error, {
    maxDepth: 3
  })}`;
  return asyncError.stack;
};

const addErrorToEachTestUnderDescribe = (describeBlock, error, asyncError) => {
  var _iterator5 = _createForOfIteratorHelper(describeBlock.tests),
      _step5;

  try {
    for (_iterator5.s(); !(_step5 = _iterator5.n()).done;) {
      const test = _step5.value;
      test.errors.push([error, asyncError]);
    }
  } catch (err) {
    _iterator5.e(err);
  } finally {
    _iterator5.f();
  }

  var _iterator6 = _createForOfIteratorHelper(describeBlock.children),
      _step6;

  try {
    for (_iterator6.s(); !(_step6 = _iterator6.n()).done;) {
      const child = _step6.value;
      addErrorToEachTestUnderDescribe(child, error, asyncError);
    }
  } catch (err) {
    _iterator6.e(err);
  } finally {
    _iterator6.f();
  }
};

exports.addErrorToEachTestUnderDescribe = addErrorToEachTestUnderDescribe;

const invariant = (condition, message) => {
  if (!condition) {
    throw new Error(message);
  }
};

exports.invariant = invariant;