"use strict";

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function _path() {
    return data;
  };

  return data;
}

function _repl() {
  const data = _interopRequireDefault(require("repl"));

  _repl = function _repl() {
    return data;
  };

  return data;
}

function _vm() {
  const data = _interopRequireDefault(require("vm"));

  _vm = function _vm() {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
let transformer;

const evalCommand = (cmd, _context, _filename, callback) => {
  let result;

  try {
    if (transformer) {
      const transformResult = transformer.process(cmd, jestGlobalConfig.replname || 'jest.js', jestProjectConfig);
      cmd = typeof transformResult === 'string' ? transformResult : transformResult.code;
    }

    result = _vm().default.runInThisContext(cmd);
  } catch (e) {
    return callback(isRecoverableError(e) ? new (_repl().default.Recoverable)(e) : e);
  }

  return callback(null, result);
};

const isRecoverableError = error => {
  if (error && error.name === 'SyntaxError') {
    return ['Unterminated template', 'Missing } in template expression', 'Unexpected end of input', 'missing ) after argument list', 'Unexpected token'].some(exception => error.message.includes(exception));
  }

  return false;
};

if (jestProjectConfig.transform) {
  let transformerPath = null;

  for (let i = 0; i < jestProjectConfig.transform.length; i++) {
    if (new RegExp(jestProjectConfig.transform[i][0]).test('foobar.js')) {
      transformerPath = jestProjectConfig.transform[i][1];
      break;
    }
  }

  if (transformerPath) {
    transformer = require(transformerPath);

    if (typeof transformer.process !== 'function') {
      throw new TypeError('Jest: a transformer must export a `process` function.');
    }
  }
}

const replInstance = _repl().default.start({
  eval: evalCommand,
  prompt: '\u203A ',
  useGlobal: true
});

replInstance.context.require = moduleName => {
  if (/(\/|\\|\.)/.test(moduleName)) {
    moduleName = _path().default.resolve(process.cwd(), moduleName);
  }

  return require(moduleName);
};