/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
'use strict'; // eslint-disable-next-line import/no-extraneous-dependencies

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const workerFarm = require('worker-farm');

const assert = require('assert');

const JestWorker = require('../../build').default; // Typical tests: node --expose-gc test.js empty 100000
//                node --expose-gc test.js loadTest 10000


assert(process.argv[2], 'Pass a child method name');
assert(process.argv[3], 'Pass the number of iteratitons');

const sleep = ms => new Promise(resolve => setTimeout(resolve, ms));

const method = process.argv[2];
const calls = +process.argv[3];
const threads = 6;

function testWorkerFarm() {
  return new Promise( /*#__PURE__*/function () {
    var _ref = _asyncToGenerator(function* (resolve, reject) {
      const startTime = Date.now();
      let count = 0;

      function countToFinish() {
        return _countToFinish.apply(this, arguments);
      }

      function _countToFinish() {
        _countToFinish = _asyncToGenerator(function* () {
          if (++count === calls) {
            workerFarm.end(api);
            const endTime = Date.now(); // Let all workers go down.

            yield sleep(2000);
            resolve({
              globalTime: endTime - startTime - 2000,
              processingTime: endTime - startProcess
            });
          }
        });
        return _countToFinish.apply(this, arguments);
      }

      const api = workerFarm({
        autoStart: true,
        maxConcurrentCallsPerWorker: 1,
        maxConcurrentWorkers: threads
      }, require.resolve('./workers/worker_farm'), [method]); // Let all workers come up.

      yield sleep(2000);
      const startProcess = Date.now();

      for (let i = 0; i < calls; i++) {
        const promisified = new Promise((resolve, reject) => {
          api[method]((err, result) => {
            if (err) {
              reject(err);
            } else {
              resolve(result);
            }
          });
        });
        promisified.then(countToFinish);
      }
    });

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }());
}

function testJestWorker() {
  return new Promise( /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator(function* (resolve, reject) {
      const startTime = Date.now();
      let count = 0;

      function countToFinish() {
        return _countToFinish2.apply(this, arguments);
      }

      function _countToFinish2() {
        _countToFinish2 = _asyncToGenerator(function* () {
          if (++count === calls) {
            farm.end();
            const endTime = Date.now(); // Let all workers go down.

            yield sleep(2000);
            resolve({
              globalTime: endTime - startTime - 2000,
              processingTime: endTime - startProcess
            });
          }
        });
        return _countToFinish2.apply(this, arguments);
      }

      const farm = new JestWorker(require.resolve('./workers/jest_worker'), {
        exposedMethods: [method],
        forkOptions: {
          execArgv: []
        },
        workers: threads
      });
      farm.getStdout().pipe(process.stdout);
      farm.getStderr().pipe(process.stderr); // Let all workers come up.

      yield sleep(2000);
      const startProcess = Date.now();

      for (let i = 0; i < calls; i++) {
        const promisified = farm[method]();
        promisified.then(countToFinish);
      }
    });

    return function (_x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }());
}

function profile(x) {
  console.profile(x);
}

function profileEnd(x) {
  console.profileEnd(x);
}

function main() {
  return _main.apply(this, arguments);
}

function _main() {
  _main = _asyncToGenerator(function* () {
    if (!global.gc) {
      console.log('GC not present');
    }

    const wFResults = [];
    const jWResults = [];

    for (let i = 0; i < 10; i++) {
      console.log('-'.repeat(75));
      profile('worker farm');
      const wF = yield testWorkerFarm();
      profileEnd('worker farm');
      yield sleep(3000); // eslint-disable-next-line no-undef

      global.gc && gc();
      profile('jest worker');
      const jW = yield testJestWorker();
      profileEnd('jest worker');
      yield sleep(3000); // eslint-disable-next-line no-undef

      global.gc && gc();
      wFResults.push(wF);
      jWResults.push(jW);
      console.log('jest-worker:', jW);
      console.log('worker-farm:', wF);
    }

    let wFGT = 0;
    let wFPT = 0;
    let jWGT = 0;
    let jWPT = 0;

    for (let i = 0; i < 10; i++) {
      wFGT += wFResults[i].globalTime;
      wFPT += wFResults[i].processingTime;
      jWGT += jWResults[i].globalTime;
      jWPT += jWResults[i].processingTime;
    }

    console.log('-'.repeat(75));
    console.log('total worker-farm:', {
      wFGT,
      wFPT
    });
    console.log('total jest-worker:', {
      jWGT,
      jWPT
    });
    console.log('-'.repeat(75));
    console.log(`% improvement over ${calls} calls (global time):`, 100 * (wFGT - jWGT) / wFGT);
    console.log(`% improvement over ${calls} calls (processing time):`, 100 * (wFPT - jWPT) / wFPT);
  });
  return _main.apply(this, arguments);
}

main();